@@ -29,14 +29,34 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 
 # Imports {{{1
+__version__ = '1.0.0'
 import re
 
 # Parameters {{{1
 CURRENCY_SYMBOLS = '$'
-DEFAULT_PRECISION = 4
+DEFAULT_HUMAN_PRECISION = 4
+DEFAULT_MACHINE_PRECISION = 8
 DEFAULT_SPACER = ''
 DEFAULT_UNITY_SCALE_FACTOR = ''
 DEFAULT_OUTPUT_SCALE_FACTORS = 'TGMkmunpfa'
+DEFAULT_IGNORE_SCALE_FACTORS = False
+DEFAULT_ASSIGNMENT_FORMATTER = '{n} = {v}'
+DEFAULT_ASSIGNMENT_RECOGNIZER = (
+    r'\A\s*(?:(\w+)\s*=\s*)?(.*?)(?:\s*--\s*(.*?)\s*)?\Z'
+)
+
+import math
+CONSTANTS = {
+    'h': (6.62606957e-34, 'J-s'),      # Plank's constant
+    'k': (1.3806488e-23, 'J/K'),       # Boltzmann's constant
+    'q': (1.602176565e-19, 'C'),       # Elementary charge
+    'c': (2.99792458e8, 'm/s'),        # Speed of light
+    'C0': (273.15, 'K'),               # Zero degrees Celsius in Kelvin
+    'eps0': (8.854187817e-12, 'F/m'),  # Permittivity of free space
+    'mu0': (4e-7*math.pi, 'H/m'),      # Permeability of free space
+    'Z0': (376.730313461, 'Ohms'),     # Characteristic impedance of free space
+}
+
 
 # Constants {{{1
 MAPPINGS = {
@@ -68,18 +88,6 @@ BIG_SCALE_FACTORS = 'kMGTPEZY'
 SMALL_SCALE_FACTORS = 'munpfazy'
     # these must be given in order, one for every three decades
 
-import math
-CONSTANTS = {
-    'h': (6.62606957e-34, 'J-s'),      # Plank's constant
-    'k': (1.3806488e-23, 'J/K'),       # Boltzmann's constant
-    'q': (1.602176565e-19, 'C'),       # Elementary charge
-    'c': (2.99792458e8, 'm/s'),        # Speed of light
-    'C0': (273.15, 'K'),               # Zero degrees Celsius in Kelvin
-    'eps0': (8.854187817e-12, 'F/m'),  # Permittivity of free space
-    'mu0': (4e-7*math.pi, 'H/m'),      # Permeability of free space
-    'Z0': (376.730313461, 'Ohms'),     # Characteristic impedance of free space
-}
-
 # Pattern Definitions {{{1
 # Build regular expressions used to recognize quantities
 def named_regex(name, regex):
@@ -174,7 +182,7 @@ simple_nan = (
     lambda match: ''
 )
 
-number_converters = [
+all_number_converters = [
     (re.compile('\A\s*{}\s*\Z'.format(pattern)), get_mant, get_sf, get_units)
     for pattern, get_mant, get_sf, get_units  in [
         number_with_exponent, number_with_scale_factor, simple_number,
@@ -183,7 +191,16 @@ number_converters = [
     ]
 ]
 
-format_spec = re.compile(r'([<>]?)(\d*)(?:\.(\d+))?([qrseEfFgG]?)'.format(**locals()))
+sf_free_number_converters = [
+    (re.compile('\A\s*{}\s*\Z'.format(pattern)), get_mant, get_sf, get_units)
+    for pattern, get_mant, get_sf, get_units  in [
+        number_with_exponent, simple_number,
+        currency_with_exponent, simple_currency,
+        nan_with_units, currency_nan, simple_nan,
+    ]
+]
+
+format_spec = re.compile(r'\A([<>]?)(\d*)(?:\.(\d+))?([qruseEfFgGdnQR]?)\Z')
 
 # Utilities {{{1
 # is_str {{{2
@@ -192,8 +209,8 @@ def is_str(obj):
     """Identifies strings in all their various guises."""
     return isinstance(obj, string_types)
 
-def to_str(num):
-    return "{0:.{1}g}".format(num, Precision+1)
+def num_to_str(num):
+    return "{0:.{1}g}".format(num, MachinePrecision+1)
 
 # _combine {{{2
 def _combine(mantissa, sf, units, spacer):
@@ -217,38 +234,79 @@ def _combine(mantissa, sf, units, spacer):
         return mantissa + sf
 
 # Preferences {{{1
-Precision = DEFAULT_PRECISION
+HumanPrecision = DEFAULT_HUMAN_PRECISION
+MachinePrecision = DEFAULT_MACHINE_PRECISION
 Spacer = DEFAULT_SPACER
 UnityScaleFactor = DEFAULT_UNITY_SCALE_FACTOR
 OutputScaleFactors = DEFAULT_OUTPUT_SCALE_FACTORS
+IgnoreScaleFactors = DEFAULT_IGNORE_SCALE_FACTORS
+AssignmentFormatter = DEFAULT_ASSIGNMENT_FORMATTER
+AssignmentRecognizer = re.compile(DEFAULT_ASSIGNMENT_RECOGNIZER)
 
-def set_preferences(prec=False, spacer=False, unity=False, output=False):
+def set_preferences(
+        hprec=False, mprec =False, spacer=False, unity=False, output=False,
+        ignore_sf=0, assign_fmt=False, assign_rec=False
+):
     """Set Global Preferences
-    prec (int): precision in digits where 0 corresponds to 1 digit, must be
-        nonnegative.
-    spacer (str): may be '' or ' ', use the latter if you prefer a space between
+
+    hprec (int): Human precision in digits where 0 corresponds to 1 digit. Must
+        be nonnegative. This precision is used when generating engineering
+        format.
+    mprec (int): Machine precision in digits where 0 corresponds to 1 digit,
+        must be nonnegative. This precision is used when not generating
+        engineering format.
+    spacer (str): May be '' or ' ', use the latter if you prefer a space between
         the number and the units. Generally using ' ' makes numbers easier to
-        use, particularly with complex units, and using '' is easier to parse.
-    unity (str): the scale factor for unity, generally '' or '_'.
-    output (str): which scale factors to output, generally one would only use
+        read, particularly with complex units, and using '' is easier to parse.
+    unity (str): The output scale factor for unity, generally '' or '_'.
+    output (str): Which scale factors to output, generally one would only use
         familiar scale factors.
+    ignore_sf (bool): Whether scale factors should be ignored by default.
+    assign_fmt (str): Format string for an assignment. Will be passed through
+        .format function. Format string takes three possible arguments named n,
+        q, and d for the name, value and description.  The default is '{n} = {v}'
+    assign_rec (str): Regular expression used to recognize an assignment. Used
+        in add_to_namespace(). Default recognizes the form
+            "Temp = 300_K -- Temperature".
 
     Any value not passed in are left alone. Pass in None to reset it to its
     default value.
     """
-    global Precision, Spacer, UnityScaleFactor, OutputScaleFactors
-    if prec is not False:
-        Precision = prec if prec is not None else DEFAULT_PRECISION
+    global HumanPrecision, MachinePrecision
+    global Spacer, UnityScaleFactor, OutputScaleFactors, IgnoreScaleFactors
+    global AssignmentFormatter, AssignmentRecognizer
+    if hprec is not False:
+        HumanPrecision = hprec if hprec is not None else DEFAULT_HUMAN_PRECISION
+    if mprec is not False:
+        MachinePrecision = (
+            mprec if mprec is not None else DEFAULT_MACHINE_PRECISION
+        )
     if spacer is not False:
         Spacer = spacer if spacer is not None else DEFAULT_SPACER
     if unity is not False:
-        UnityScaleFactor = unity if unity is not None else DEFAULT_UNITY_SCALE_FACTOR
+        UnityScaleFactor = (
+            unity if unity is not None else DEFAULT_UNITY_SCALE_FACTOR
+        )
     if output is not False:
-        OutputScaleFactors = output if output is not None else DEFAULT_OUTPUT_SCALE_FACTORS
+        OutputScaleFactors = (
+            output if output is not None else DEFAULT_OUTPUT_SCALE_FACTORS
+        )
+    if ignore_sf is not 0:
+        IgnoreScaleFactors = (
+            ignore_sf if ignore_sf is not None else DEFAULT_IGNORE_SCALE_FACTORS
+        )
+    if assign_fmt is not False:
+        AssignmentFormatter = (
+            assign_fmt if assign_fmt is not None else DEFAULT_ASSIGNMENT_FORMATTER
+        )
+    if assign_rec is not False:
+        AssignmentRecognizer = re.compile(
+            assign_rec if assign_rec is not None else DEFAULT_ASSIGNMENT_RECOGNIZER
+        )
 
 # Quantity class {{{1
 class Quantity:
-    def __init__(self, value, units=None):
+    def __init__(self, value, units=None, ignore_sf=None):
         """Physical Quantity
         A real quantity with units.
 
@@ -260,8 +318,14 @@ class Quantity:
         if units is None:
             units = ''
         self._value = value
-        self._units = units
+        self.units = units
+        ignore_sf = IgnoreScaleFactors if ignore_sf is None else ignore_sf
+
         if is_str(value):
+            if ignore_sf:
+                number_converters = sf_free_number_converters
+            else:
+                number_converters = all_number_converters
             # if we get a string, keep all the pieces so we can reconstruct it
             # exactly as it was given.
             for pattern, get_mant, get_sf, get_units in number_converters:
@@ -271,15 +335,15 @@ class Quantity:
                     self._mantissa = get_mant(match)
                     sf = get_sf(match)
                     self._scale_factor = sf if sf != '_' else ''
-                    self._units = get_units(match)
-                    if self._units:
+                    self.units = get_units(match)
+                    if self.units:
                         if units:
-                            assert units == self._units
+                            assert units == self.units
                     else:
-                        self._units = units
+                        self.units = units
                     return
             try:
-                self._value, self._units = CONSTANTS[value]
+                self._value, self.units = CONSTANTS[value]
             except KeyError:
                 raise ValueError('%s: not a valid number.' % value)
 
@@ -291,11 +355,15 @@ class Quantity:
         value = self._mantissa if self._value is None else str(self._value)
         return value.lower() in ['nan', '-nan', '+nan']
 
-    def units(self):
-        """Returns the units."""
-        return self._units
+    def add_name(self, name):
+        """Add a name."""
+        self.name = name
 
-    def strip_units(self):
+    def add_desc(self, desc):
+        """Add a description."""
+        self.desc = desc
+
+    def strip(self):
         """Returns the value as a string in the originally given notation.
 
         The original string is returned with units removed. This allows you
@@ -305,9 +373,9 @@ class Quantity:
         if self._value is None:
             return self._mantissa + self._scale_factor
         else:
-            return to_str(self._value)
+            return num_to_str(self._value)
 
-    def to_number(self):
+    def to_float(self):
         """Returns the value as a float."""
         if self._value is None:
             sf = self._scale_factor
@@ -315,7 +383,7 @@ class Quantity:
         else:
             return self._value
 
-    def to_flt_number(self):
+    def to_unitless_str(self):
         """Renders the value as a string in floating point notation.
 
         The original string is returned with units removed and the scale factor
@@ -327,44 +395,44 @@ class Quantity:
             sf = self._scale_factor
             return self._mantissa + MAPPINGS.get(sf, [sf])[0]
         else:
-            return to_str(self._value)
+            return num_to_str(self._value)
 
-    def to_eng_number(self, prec=None):
+    def to_unitless_eng(self, prec=None):
         """Renders the value as a string in engineering notation."""
         # this is a bit of a hack, temporarily remove the units
-        units = self.units()
-        self._units = None
-        eng_number = self.to_eng_quantity(prec)
-        self._units = units
+        units = self.units
+        self.units = None
+        eng_number = self.to_eng(prec)
+        self.units = units
         return eng_number
 
-    def to_quantity(self):
+    def to_tuple(self):
         """Returns a tuple that contains the value as a float and the units."""
-        return self.to_number(), self.units()
+        return self.to_float(), self.units
 
-    def to_flt_quantity(self):
+    def to_str(self):
         """Renders the value and units as a string in floating point notation."""
-        number = self.to_flt_number()
-        units = self.units()
+        number = self.to_unitless_str()
+        units = self.units
         return _combine(number, '', units, Spacer)
 
-    def to_eng_quantity(self, prec=None):
+    def to_eng(self, prec=None):
         """Renders the value and units as a string in engineering notation."""
 
         # determine precision
         if prec is None:
-            prec = Precision
+            prec = HumanPrecision
         else:
             prec = int(prec)
         assert (prec >= 0)
 
         # check for infinities or NaN
         if self.is_infinite() or self.is_nan():
-            return _combine(self.strip_units(), '', self.units(), ' ')
+            return _combine(self.strip(), '', self.units, ' ')
 
         # convert into scientific notation with proper precision
-        value = self.to_number()
-        units = self.units()
+        value = self.to_float()
+        units = self.units
         number = "%.*e" % (prec, value)
         mantissa, exp = number.split("e")
         exp = int(exp)
@@ -407,61 +475,95 @@ class Quantity:
         return _combine(mantissa, sf, units, Spacer)
 
     def __str__(self):
-        return self.to_eng_quantity()
+        return self.to_eng()
 
     def __format__(self, fmt):
         """Convert quantity to string for Python string format function.
 
         Supports the normal floating point and string format types as well
-        'q' and 'r'.  All will output the number using the SI scale factors, but
-        the 's' and 'q' types also include the units.
+        'q', 'r' and 'u'.  All will output the number using the SI scale
+        factors, but the 's' and 'q' types also include the units.
 
         The format is specified using AW.PT where:
         A is character and gives the alignment: either '', '>', or '<'
         W is integer and gives the width
         P is integer and gives the precision
-        T is  and gives the type: choose from q, r, s, e, D, f, F, g, G
+        T is  and gives the type: choose from q, r, s, e, f, F, g, G, u, n, ...
+           q = quantity (1.4204GHz)
+           r = real (1.4204G)
+           s = string (1.4204GHz)
+           e = exponential form (1.4204e9)
+           f, F = float (1420400000.000000)
+           g, G = float (1.4204e+09)
+           u = units (Hz)
+           n = name (f)
+           d = description (hydrogen line)
+           Q = name and quantity (f = 1.4204GHz)
+           R = name and real (f = 1.4204G)
         """
         match = format_spec.match(fmt)
         if match:
             align, width, prec, ftype = match.groups()
             if ftype in 'qs':
-                value = self.to_eng_quantity(prec)
-                return '{0:{1}{2}s}'.format(value, align, width, value)
+                value = self.to_eng(prec)
+                return '{0:{1}{2}s}'.format(value, align, width)
             elif ftype == 'r':
-                value = self.to_eng_number(prec)
-                return '{0:{1}{2}s}'.format(value, align, width, value)
+                value = self.to_unitless_eng(prec)
+                return '{0:{1}{2}s}'.format(value, align, width)
+            elif ftype == 'u':
+                value = self.units
+                return '{0:{1}{2}s}'.format(value, align, width)
+            elif ftype == 'n':
+                value = getattr(self, 'name', '')
+                return '{0:{1}{2}s}'.format(value, align, width)
+            elif ftype == 'd':
+                value = getattr(self, 'desc', '')
+                return '{0:{1}{2}s}'.format(value, align, width)
+            elif ftype in 'Q':
+                name = getattr(self, 'name', '')
+                desc = getattr(self, 'desc', '')
+                value = self.to_eng(prec)
+                if name:
+                    value = AssignmentFormatter.format(n=name, v=value, d=desc)
+                return '{0:{1}{2}s}'.format(value, align, width)
+            elif ftype in 'R':
+                name = getattr(self, 'name', '')
+                desc = getattr(self, 'desc', '')
+                value = self.to_unitless_eng(prec)
+                if name:
+                    value = AssignmentFormatter.format(n=name, v=value, d=desc)
+                return '{0:{1}{2}s}'.format(value, align, width)
             else:
-                value = self.to_number()
+                value = self.to_float()
                 return '{0:{1}}'.format(value, fmt)
         else:
-            return self.to_eng_quantity()
+            return self.to_eng()
 
     def __float__(self):
-        return self.to_number()
+        return self.to_float()
 
 
 # Shortcut functions {{{1
-def to_quantity(value, units=None):
-    return Quantity(value, units).to_quantity()
+def quant_to_tuple(value, units=None):
+    return Quantity(value, units).to_tuple()
 
-def to_eng_quantity(value, units=None, prec=None):
-    return Quantity(value, units).to_eng_quantity(prec)
+def quant_to_eng(value, units=None, prec=None):
+    return Quantity(value, units).to_eng(prec)
 
-def to_flt_quantity(value, units=None):
-    return Quantity(value, units).to_flt_quantity()
+def quant_to_str(value, units=None):
+    return Quantity(value, units).to_str()
 
-def to_number(value, units=None):
-    return Quantity(value, units).to_number()
+def quant_to_float(value, units=None):
+    return Quantity(value, units).to_float()
 
-def to_eng_number(value, units=None, prec=None):
-    return Quantity(value, units).to_eng_number(prec)
+def quant_to_unitless_eng(value, units=None, prec=None):
+    return Quantity(value, units).to_unitless_eng(prec)
 
-def to_flt_number(value, units=None):
-    return Quantity(value, units).to_flt_number()
+def quant_to_unitless_str(value, units=None):
+    return Quantity(value, units).to_unitless_str()
 
-def strip_units(value):
-    return Quantity(value).strip_units()
+def quant_strip(value):
+    return Quantity(value).strip()
 
 # Text processing functions {{{1
 # All to engineering format {{{2
@@ -477,8 +579,8 @@ def all_to_eng_fmt(text):
         end = match.start(0)
         number = match.group(0)
         try:
-            number = to_eng_quantity(number)
-        except ValueError:
+            number = quant_to_eng(number)
+        except ValueError: # pragma: no cover
             # something unexpected happened
             # but this is not essential, so ignore it
             pass
@@ -500,8 +602,8 @@ def all_from_eng_fmt(text):
         end = match.start(0)
         number = match.group(0)
         try:
-            number = to_flt_quantity(number)
-        except ValueError:
+            number = quant_to_str(number)
+        except ValueError: # pragma: no cover
             # something unexpected happened
             # but this is not essential, so ignore it
             pass
@@ -534,7 +636,11 @@ def add_to_namespace(quantities):
             name, value, desc = match.groups()
             if not value:
                 continue
+            if not name:
+                raise ValueError('{}: no variable name given.'.format(line))
             quantity = Quantity(value)
+            quantity.add_name(name)
+            quantity.add_desc(desc)
             namespace[name] = quantity
-        else:
+        else: # pragma: no cover
             raise ValueError('{}: not a valid number.'.format(line))
